{% load crispy_forms_tags %}

<!-- Relaxation Info -->
<div class="accordion-item">
    <h2 class="accordion-header" id="headingRelaxFilters">
        <button class="accordion-button collapsed" type="button" data-bs-toggle="collapse"
            data-bs-target="#collapseRelaxFilters" aria-expanded="true" aria-controls="collapseRelaxFilters">
            <b>By Relaxation Info:</b>
        </button>
    </h2>
    <div id="collapseRelaxFilters" class="accordion-collapse collapse" aria-labelledby="headingRelaxFilters">
        <div class="accordion-body">
            {{ form.volume_change__range | as_crispy_field }}
        </div>
    </div>
</div>

{% include "core_components/base_filter_types/calculation.html" %}
{% include "core_components/base_filter_types/structure.html" %}

<!-- Electronic Info -->
<div class="accordion-item">
    <h2 class="accordion-header" id="headingElectronicFilters">
        <button class="accordion-button collapsed" type="button" data-bs-toggle="collapse"
            data-bs-target="#collapseElectronicFilters" aria-expanded="true" aria-controls="collapseElectronicFilters">
            <b>By Electronic Info:</b>
        </button>
    </h2>
    <div id="collapseElectronicFilters" class="accordion-collapse collapse" aria-labelledby="headingElectronicFilters">
        <div class="accordion-body">
            <a><em><b>WARNING:</b> even though some relaxations provide this information, these values are
                    very rough estimates.</em></a>

            {{ form.band_gap__range | as_crispy_field }}
            {{ form.is_gap_direct | as_crispy_field }}
            {{ form.energy_fermi__range | as_crispy_field }}
            {{ form.conduction_band_minimum__range | as_crispy_field }}
            {{ form.valence_band_maximum__range | as_crispy_field }}
        </div>
    </div>
</div>