<div class="container p-4">
    <h2>Band Structure Calculation Results</h2>

    {% include "core_components/base_data_types/calculation.html" %}
    {% include "core_components/base_data_types/structure.html" %}
    {% include "core_components/base_data_types/density-of-states.html" %}

</div>