# -*- coding: utf-8 -*-

from simmate.database.base_data_types import StaticEnergy


class MITStaticEnergy(StaticEnergy):
    pass


class MatprojStaticEnergy(StaticEnergy):
    pass


class NEBEndpointStaticEnergy(StaticEnergy):
    pass


class Quality04StaticEnergy(StaticEnergy):
    pass
