{% include "core_components/base_filter_types/calculation.html" %}
{% include "core_components/base_filter_types/structure.html" %}
{% include "core_components/base_filter_types/density-of-states.html" %}