<footer class="footer">
    <div class="container-fluid">
        <div class="row">
            <div class="col-md-6">
                Simmate <script>document.write(new Date().getFullYear())</script>
            </div>
            <div class="col-md-6">
                <div class="text-md-end footer-links d-none d-md-block">
                    <a href="{% url 'about' %}">About</a>
                    <a href="{% url 'contact' %}">Contact Us</a>
                </div>
            </div>
        </div>
    </div>
</footer>