# -*- coding: utf-8 -*-

"""
This file is required!

You typically don't have anything extra other than what is shown below. All this
does is tell django where to find your code.
"""

from django.apps import AppConfig


class ExampleAppConfig(AppConfig):
    name = "example_app"
