# -*- coding: utf-8 -*-

"""
There are not any custom inputs for Bader analysis. The inputs required
are actually outputs from VASP or some other DFT code. This module lists out
some of those file types for convenience.
"""

# TODO these files don't exist yet
# from simmate.calculators.vasp.outputs import CHGCAR, AECCAR0, AECCAR2
