{% extends "core_components/site_base.html" %}

{% block tabtitle %}
Simmate
{% endblock %}

{% block banner %}
{% include "core_components/header.html"%}
{% endblock %}


{% block body %}

<!-- NOTE: for this section to work, be sure to read datatable.html -->


<div class="container p-4 bg-light border" id="search-results">

  <!-- Breadcrumb links -->
  <nav aria-label="breadcrumb">
    <ol class="breadcrumb">
      <li class="breadcrumb-item"><a href="{% url 'workflows' %}">workflows</a></li>
      <li class="breadcrumb-item"><a href="{% url 'workflows_by_type' workflow.name_project %}">{{ workflow.name_project }}</a></li>
      <li class="breadcrumb-item"><a href="{% url 'workflow_detail' workflow.name_project workflow.name_preset %}">
          {{ workflow.name_preset }}</a></li>
      <li class="breadcrumb-item active">submit</li>
    </ol>
  </nav>

  <!-- Submission form -->
  <form action="{% url 'workflow_submit' workflow.name_project workflow.name_preset %}" method="post"
    enctype="multipart/form-data">
    {% csrf_token %}
    {% load crispy_forms_tags %}
    {{ submission_form | crispy }}
    <input type="submit" value="Submit">
  </form>

</div>

{% endblock %}

{% block extrafooter %}
{% include "core_components/datatable.html" %}
{% endblock %}