{% extends "core_components/site_base.html" %}

{% block tabtitle %}
Simmate
{% endblock %}

{% block banner %}
{% include "core_components/header.html"%}
{% endblock %}


{% block body %}

<!-- start page title -->
<div class="row">
  <div class="col-12">
    <div class="page-title-box">
      <div class="page-title-right">
        <ol class="breadcrumb m-0">
          <li class="breadcrumb-item"><a href="{% url 'workflows' %}">Workflows</a></li>
          <li class="breadcrumb-item active">{{ workflow_type }}</li>
        </ol>
      </div>
      <h4 class="page-title">Workflow Type</h4>
    </div>
  </div>
</div>
<!-- end page title -->

<!-- Header -->
<section class="pt-5 jumbotron text-center">
  <div class="container">
    <h1 class="jumbotron-heading">{{ workflow_type }}</h1>
    <p class="lead text-muted">{{ workflow_type_description_short }}</p>
  </div>
</section>

<!-- List of Cards -->
<div class="row">
  {% for workflow in workflows %}
  <div class="col-md-4">
    <div class="card border-primary border">
      <div class="card-body">
        <h5 class="card-title text-primary">{{ workflow.name_preset }}</h5>
        <!-- should I wrap the markdown in <p class="card-text"></p> ? -->
        {{ workflow.description_doc_short }}
        <br><br>
        <a href="{% url 'workflow_detail' workflow_type workflow.name_preset %}" class="btn btn-primary btn-sm">About</a>
      </div>
    </div>
  </div>
  {% endfor %}
</div>

{% endblock %}