<div class="container p-4">
    <h2>Diffusion Analysis Results</h2>
    <h4>NOTE: Rendered results from a diffusion analysis are not yet available. Please use the API/JSON formats or
        python in order to view results.</h4>
</div>