{% load crispy_forms_tags %}

<!-- Diffusion Info -->
<div class="accordion-item">
    <h2 class="accordion-header" id="headingDiffusionFilters">
        <button class="accordion-button collapsed" type="button" data-bs-toggle="collapse"
            data-bs-target="#collapseDiffusionFilters" aria-expanded="true" aria-controls="collapseDiffusionFilters">
            <b>By Diffusion Info:</b>
        </button>
    </h2>
    <div id="collapseDiffusionFilters" class="accordion-collapse collapse" aria-labelledby="headingDiffusionFilters">
        <div class="accordion-body">
            {{ form.migrating_specie | as_crispy_field }}
            {{ form.vacancy_mode | as_crispy_field }}
            {{ form.atomic_fraction__range | as_crispy_field }}
            {{ form.barrier_cell__range | as_crispy_field }}
            {{ form.npaths_involved__range | as_crispy_field }}
        </div>
    </div>
</div>

{% include "core_components/base_filter_types/structure.html" %}