{% extends "core_components/site_base.html" %}

{% block tabtitle %}
Simmate
{% endblock %}

{% block banner %}
{% include "core_components/header.html"%}
{% endblock %}


{% block body %}

<!-- Header -->
<section class="pt-5 jumbotron text-center">
  <div class="container">
    <h1 class="jumbotron-heading">Third-Party Data</h1>
    <p class="lead text-muted">Explore some of the most popular material databases and immediately jump to the
      provider's website for more data</p>
  </div>
</section>

<!-- List of Cards -->
<div class="row">
  {% for workflow_type, workflow_description in workflows_metadata.items %}
  <div class="col-md-4">
    <div class="card border-primary border">
      <div class="card-body">
        <h5 class="card-title text-primary">{{ workflow_type }}</h5>
        <p class="card-text">{{ workflow_description }}</p>
        <a href="{% url 'third_parties:provider' workflow_type %}" class="btn btn-primary btn-sm">About</a>
      </div>
    </div>
  </div>
  {% endfor %}
</div>

{% endblock %}