# -*- coding: utf-8 -*-

"""
This file is only required for editable installs using pip. We can
remove this file once the following issue is closed:
  https://github.com/pypa/setuptools/issues/2816

As extra, the example_project/setup.py can also be converted to pyproject.toml
and removed. I hold off on this to minimize new files made for users.
"""

from setuptools import setup

setup()
